<?php

namespace SM\Factory;

use Doctrine\ORM\EntityManager;
use \Teknisa\Model\Traits\TQuery;

class ComposicaoServicosFactory
{
    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }
    
    public function getComposicaoServicos($filtroPrimeiraLinha, $filtroNome, $filtroIdAtivo, $filtroIdCompObrigatoria, $filtroNrCompServico, $filtroDsCompServico, $filtroCdCentCust, $filtroUtilizaRevenda)
    {
        $sql = "SELECT
                    COMP.NRCOMPSERVICO,
                    COMP.NMCOMPSERVICO,
                    COMP.DSCOMPSERVICO,
                    COMP.NRCOMPSERVICOPAI,
                    PAI.NMCOMPSERVICO AS NMCOMPSERVICOPAI,
                    COMP.IDCOMPOBRIGATORIA,
                    COMP.NRORG,
                    COMP.DTINCLUSAO,
                    COMP.DTULTATU,
                    COMP.NRORGINCLUSAO,
                    COMP.CDOPERINCLUSAO,
                    INC.NMOPERADOR    AS NMOPERINCLUSAO,
                    COMP.NRORGULTATU,
                    COMP.CDOPERULTATU,
                    ALT.NMOPERADOR    AS NMOPERULTATU,
                    COMP.CDCENTCUST,
                    COMP.IDUTILIZAREVENDA,
                    CUSTO.NMCENTCUST,
                    COMP.IDATIVO,
                    DECODE((SELECT COUNT(*) AS QTFILHOS
				            FROM FCT_COMPSERVICO COMPSERV 
						   WHERE COMPSERV.NRCOMPSERVICOPAI = COMP.NRCOMPSERVICO), 0, 0, 1) AS HASCHILD
                    FROM
                    FCT_COMPSERVICO COMP
                    LEFT JOIN FCT_COMPSERVICO PAI ON COMP.NRCOMPSERVICOPAI = PAI.NRCOMPSERVICO
                    LEFT JOIN OPERADOR        INC ON COMP.CDOPERINCLUSAO = INC.CDOPERADOR
                    LEFT JOIN OPERADOR        ALT ON COMP.CDOPERULTATU = ALT.CDOPERADOR
                    LEFT JOIN CENTCUST        CUSTO ON COMP.CDCENTCUST = CUSTO.CDCENTCUST
                    $filtroPrimeiraLinha
                    $filtroNome
                    $filtroIdAtivo
                    $filtroIdCompObrigatoria
                    $filtroNrCompServico
                    $filtroDsCompServico
                    $filtroCdCentCust
                    $filtroUtilizaRevenda
                ORDER BY NMCOMPSERVICO ASC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }
    public function getComposicaoServicosAtivo($filtroPrimeiraLinha, $filtroNome, $filtroIdCompObrigatoria, $filtroNrCompServico, $filtroDsCompServico, $filtroCdCentCust, $filtroUtilizaRevenda)
    {
        $sql = "SELECT
                    COMP.NRCOMPSERVICO,
                    COMP.NMCOMPSERVICO,
                    COMP.DSCOMPSERVICO,
                    COMP.NRCOMPSERVICOPAI,
                    PAI.NMCOMPSERVICO AS NMCOMPSERVICOPAI,
                    COMP.IDCOMPOBRIGATORIA,
                    COMP.NRORG,
                    CASE 
                        WHEN COMP.IDATIVO = 'S' THEN 'Sim'
                        WHEN COMP.IDATIVO = 'N' THEN 'Não'
                        ELSE COMP.IDATIVO
                    END AS IDATIVO,
                    COMP.DTINCLUSAO,
                    COMP.DTULTATU,
                    COMP.NRORGINCLUSAO,
                    COMP.CDOPERINCLUSAO,
                    INC.NMOPERADOR AS NMOPERINCLUSAO,
                    COMP.NRORGULTATU,
                    COMP.CDOPERULTATU,
                    ALT.NMOPERADOR AS NMOPERULTATU,
                    COMP.CDCENTCUST,
                    COMP.IDUTILIZAREVENDA,
                    CUSTO.NMCENTCUST,
                    DECODE((SELECT COUNT(*) AS QTFILHOS
                            FROM FCT_COMPSERVICO COMPSERV 
                           WHERE COMPSERV.NRCOMPSERVICOPAI = COMP.NRCOMPSERVICO), 0, 0, 1) AS HASCHILD
                FROM
                    FCT_COMPSERVICO COMP
                LEFT JOIN FCT_COMPSERVICO PAI ON COMP.NRCOMPSERVICOPAI = PAI.NRCOMPSERVICO
                LEFT JOIN OPERADOR        INC ON COMP.CDOPERINCLUSAO = INC.CDOPERADOR
                LEFT JOIN OPERADOR        ALT ON COMP.CDOPERULTATU = ALT.CDOPERADOR
                LEFT JOIN CENTCUST        CUSTO ON COMP.CDCENTCUST = CUSTO.CDCENTCUST
                $filtroPrimeiraLinha
                $filtroNome
                AND COMP.IDATIVO IN ('S', 'N')
                $filtroIdCompObrigatoria
                $filtroNrCompServico
                $filtroDsCompServico
                $filtroCdCentCust
                $filtroUtilizaRevenda
                ORDER BY NMCOMPSERVICO ASC";
    
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }    

    public function getItComposicaoServico($params)
    {
        $sql = "SELECT
                    IT.NRITCOMPSERVICO,
                    IT.NRCOMPSERVICO,
                    IT.CDSERVORCA,
                    IT.IDSOMAVALORSUP,
                    IT.IDSERVOBRIGCOMP,
                    IT.NRORG,
                    IT.IDATIVO,
                    IT.DTINCLUSAO,
                    IT.DTULTATU,
                    IT.NRORGINCLUSAO,
                    IT.CDOPERINCLUSAO,
                    INC.NMOPERADOR AS NMOPERINCLUSAO,
                    IT.NRORGULTATU,
                    IT.CDOPERULTATU,
                    ALT.NMOPERADOR AS NMOPERULTATU,
                    IT.IDELEMENTO,
                    ELEM.NMSERVORCA,
                    ELEM.IDORDENAELEMENTO,
                    IT.IDTIPOELEMENTO,
                    IT.IDELEMENTOPAI,
                    PAI.NMSERVORCA AS NMSERVORCAPAI,
                    IT.IDAPARECEPROPOSTA,
                    ELEM.CDSERVORCA || IT.IDELEMENTO AS IDENTIFICADOR,
                    PAI.CDSERVORCA || IT.IDELEMENTOPAI AS IDENTIFICADORPAI
                FROM
                    FCT_ITCOMPSERVICO IT
                    LEFT JOIN FCT_SERVICOORCA   ELEM ON IT.IDELEMENTO = ELEM.IDELEMENTO
                    LEFT JOIN FCT_SERVICOORCA   PAI ON IT.IDELEMENTOPAI = PAI.IDELEMENTO
                    LEFT JOIN OPERADOR          INC ON IT.CDOPERINCLUSAO = INC.CDOPERADOR
                    LEFT JOIN OPERADOR          ALT ON IT.CDOPERULTATU = ALT.CDOPERADOR
                WHERE
                        ELEM.IDATIVO <> 'N'
                    AND NRCOMPSERVICO = :NRCOMPSERVICO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    } 

    public function getServicoOrcaTelaElemento()
    {
        $sql = "SELECT
                    ELEMENTO.DSEXPRESSAOCALC,
                    ELEMENTO.IDTIPOCALCULO,
                    ELEMENTO.VRPERCCALCULO,
                    ELEMENTO.IDALTERAFORMPREC,
                    ELEMENTO.CDSERVORCA,
                    ELEMENTO.CDFILIAL,
                    ELEMENTO.IDITEMCALCULO,
                    ELEMENTO.NMSERVORCA,
                    ELEMENTO.DSSERVORCA,
                    ELEMENTO.CDPRODFATSERV,
                    ELEMENTO.IDATIVO,
                    ELEMENTO.IDTIPO,
                    ELEMENTO.IDELEMENTO,
                    ELEMENTO.IDTIPOELEMENTO,
                    ELEMENTO.IDELEMENTOPAI,
                    PAI.NMSERVORCA                                                     AS NMSERVORCAPAI,
                    ELEMENTO.NRORG,
                    ELEMENTO.DTINCLUSAO,
                    ELEMENTO.DTULTATU,
                    ELEMENTO.NRORGINCLUSAO,
                    ELEMENTO.CDOPERINCLUSAO,
                    OPINC.NMOPERADOR                                                   AS NMOPERINCLUSAO,
                    ELEMENTO.NRORGULTATU,
                    ELEMENTO.CDOPERULTATU,
                    OPATU.NMOPERADOR                                                   AS NMOPERULTATU,
                    DECODE(ELEMENTO.IDELEMENTOPAI, NULL, 0, ELEMENTO.IDORDENAELEMENTO) AS IDORDENAELEMENTO_TELA,
                    ELEMENTO.IDORDENAELEMENTO,
                    ELEMENTO.CDTPMEDIDAVARIAVEL,
                    MEDIDA.DSTPMEDIDASERV,
                    DECODE(NVL(ELEMENTO.IDELEMENTO, 'G_' || ELEMENTO.CDSERVORCA),
                        'G_' || ELEMENTO.CDSERVORCA,
                        'N',
                        'S')                                                        AS IDBLOQUEADO,
                    ELEMENTO.IDFORMACAOPRECO,
                    ELEMENTO.CDCENTCUST,
                    CUSTO.NMCENTCUST,
                    ELEMENTO.CDSERVORCA || ELEMENTO.IDELEMENTO AS IDENTIFICADOR,
                    PAI.CDSERVORCA || ELEMENTO.IDELEMENTOPAI AS IDENTIFICADORPAI
                FROM
                    FCT_SERVICOORCA    ELEMENTO
                    LEFT JOIN FCT_SERVICOORCA    PAI ON ELEMENTO.IDELEMENTOPAI = PAI.IDELEMENTO
                    LEFT JOIN FCT_TIPOMEDIDASERV MEDIDA ON ELEMENTO.CDTPMEDIDAVARIAVEL = MEDIDA.CDTPMEDIDASERV
                    LEFT JOIN OPERADOR           OPINC ON ELEMENTO.CDOPERINCLUSAO = OPINC.CDOPERADOR
                    LEFT JOIN OPERADOR           OPATU ON ELEMENTO.CDOPERULTATU = OPATU.CDOPERADOR
                    LEFT JOIN CENTCUST           CUSTO ON ELEMENTO.CDCENTCUST = CUSTO.CDCENTCUST
                WHERE
                    ELEMENTO.IDTIPO = 'E'
                    AND ELEMENTO.IDATIVO = 'S'
                    ORDER BY NMSERVORCA ASC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getElementoServicoSuperior($params)
    {
        $sql = "SELECT
                    IT.NRITCOMPSERVICO,
                    IT.NRCOMPSERVICO,
                    IT.CDSERVORCA,
                    IT.IDSOMAVALORSUP,
                    IT.IDSERVOBRIGCOMP,
                    IT.NRORG,
                    IT.IDATIVO,
                    IT.DTINCLUSAO,
                    IT.DTULTATU,
                    IT.NRORGINCLUSAO,
                    IT.CDOPERINCLUSAO,
                    INC.NMOPERADOR AS NMOPERINCLUSAO,
                    IT.NRORGULTATU,
                    IT.CDOPERULTATU,
                    ALT.NMOPERADOR AS NMOPERULTATU,
                    IT.IDELEMENTO,
                    ELEM.NMSERVORCA,
                    ELEM.IDORDENAELEMENTO,
                    IT.IDTIPOELEMENTO,
                    IT.IDELEMENTOPAI,
                    PAI.NMSERVORCA AS NMSERVORCAPAI,
                    IT.IDAPARECEPROPOSTA,
                    IT.CDSERVORCA || ELEM.IDELEMENTO AS IDENTIFICADOR,
                    PAI.CDSERVORCA || ELEM.IDELEMENTOPAI AS IDENTIFICADORPAI
                FROM
                    FCT_ITCOMPSERVICO IT
                    LEFT JOIN FCT_SERVICOORCA   ELEM ON IT.IDELEMENTO = ELEM.IDELEMENTO
                    LEFT JOIN FCT_SERVICOORCA   PAI ON IT.IDELEMENTOPAI = PAI.IDELEMENTO
                    LEFT JOIN OPERADOR          INC ON IT.CDOPERINCLUSAO = INC.CDOPERADOR
                    LEFT JOIN OPERADOR          ALT ON IT.CDOPERULTATU = ALT.CDOPERADOR
                WHERE
                        ELEM.IDATIVO <> 'N'
                    AND NRCOMPSERVICO = :NRCOMPSERVICO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function insertComposicaoServicos($params)
    {
        $sql = "INSERT INTO FCT_COMPSERVICO
                        (NRCOMPSERVICO,
                        NMCOMPSERVICO,
                        DSCOMPSERVICO,
                        NRCOMPSERVICOPAI,
                        IDCOMPOBRIGATORIA,
                        CDCENTCUST,
                        IDUTILIZAREVENDA,
                        IDATIVO,
                        NRORG,
                        NRORGINCLUSAO,
                        NRORGULTATU,
                        CDOPERINCLUSAO,
                        CDOPERULTATU,
                        DTINCLUSAO,
                        DTULTATU)
                VALUES (:NRCOMPSERVICO,
                        :NMCOMPSERVICO,
                        :DSCOMPSERVICO,
                        :NRCOMPSERVICOPAI,
                        :IDCOMPOBRIGATORIA,
                        :CDCENTCUST,
                        :IDUTILIZAREVENDA,
                        :IDATIVO,
                        :NRORG,
                        :NRORG,
                        :NRORG,
                        :CDOPERINCLUSAO,
                        :CDOPERINCLUSAO,
                        SYSDATE,
                        SYSDATE)";
        $this->connection->executeStatement($sql, $params);
    }

    /**
     * Replica o IDATIVO para todos os filhos da composição de serviço alterada
    **/
    public function replicarAlteracaoIdAtivo(array $params){
        
        $sql = "UPDATE FCT_COMPSERVICO
                SET IDATIVO             = :IDATIVO,
                    CDOPERULTATU        = :CDOPERADOR,
                    NRORGULTATU         = :NRORG,
                    CDCENTCUST          = :CDCENTCUST,
                    IDUTILIZAREVENDA    = :IDUTILIZAREVENDA
                WHERE NRCOMPSERVICO IN(SELECT NRCOMPSERVICO
                                          FROM FCT_COMPSERVICO
                                          CONNECT BY PRIOR NRCOMPSERVICO = NRCOMPSERVICOPAI
                                          START WITH NRCOMPSERVICO       = :NRCOMPSERVICO)";

        $this->connection->executeStatement($sql, $params);
    }

    public function updateComposicaoServicos($params)
    {
        $sql = "UPDATE FCT_COMPSERVICO
                   SET 
                        NMCOMPSERVICO       = :NMCOMPSERVICO,
                        DSCOMPSERVICO       = :DSCOMPSERVICO,
                        IDATIVO             = :IDATIVO,
                        IDCOMPOBRIGATORIA   = :IDCOMPOBRIGATORIA,
                        NRCOMPSERVICOPAI    = :NRCOMPSERVICOPAI,
                        CDCENTCUST          = :CDCENTCUST,
                        IDUTILIZAREVENDA    = :IDUTILIZAREVENDA,
                        NRORG               = :NRORG,
                        NRORGULTATU         = :NRORG,
                        CDOPERULTATU        = :CDOPERULTATU,
                        DTULTATU            = SYSDATE
                WHERE NRCOMPSERVICO   = :NRCOMPSERVICO";

        $this->connection->executeStatement($sql, $params);
    }

    /**
     * Exclui a composição e seus filhos.
     * 
     */
    public function deleteComposicaoEmCascata($params) {
        $sql = "DELETE FROM FCT_COMPSERVICO
                 WHERE NRCOMPSERVICO IN (SELECT NRCOMPSERVICO
                                          FROM FCT_COMPSERVICO
                                         START WITH NRCOMPSERVICO     = :NRCOMPSERVICO
                                       CONNECT BY PRIOR NRCOMPSERVICO = NRCOMPSERVICOPAI)";
        
        $this->connection->executeStatement($sql, $params);
    }

    public function verificarItemElementoRepetido($params)
    {
        $sql = "SELECT COUNT(*) AS EXISTE
                    FROM FCT_ITCOMPSERVICO
                WHERE NRCOMPSERVICO = :NRCOMPSERVICO
                    AND IDELEMENTO    = :IDELEMENTO";

        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    public function insertElementoServico($params)
    {
        $sql = "INSERT INTO FCT_ITCOMPSERVICO
                        (NRITCOMPSERVICO,
                        NRCOMPSERVICO,
                        CDSERVORCA,
                        IDSOMAVALORSUP,
                        IDSERVOBRIGCOMP,
                        NRORG,
                        IDATIVO,
                        DTINCLUSAO,
                        DTULTATU,
                        NRORGINCLUSAO,
                        CDOPERINCLUSAO,
                        NRORGULTATU,
                        CDOPERULTATU,
                        IDELEMENTO,
                        IDELEMENTOPAI,
                        IDTIPOELEMENTO,
                        IDAPARECEPROPOSTA)
                VALUES (:NRITCOMPSERVICO,
                        :NRCOMPSERVICO,
                        :CDSERVORCA,
                        :IDSOMAVALORSUP,
                        :IDSERVOBRIGCOMP,
                        :NRORG,
                        :IDATIVO,
                        SYSDATE,
                        SYSDATE,
                        :NRORG,
                        :CDOPERINCLUSAO,
                        :NRORG,
                        :CDOPERINCLUSAO,
                        :IDELEMENTO,
                        :IDELEMENTOPAI,
                        :IDTIPOELEMENTO,
                        :IDAPARECEPROPOSTA)";
        $this->connection->executeStatement($sql, $params);
    }

    /**
     * Replica a obrigatoriedade do item da composição para seus itens filhos.
     */
    public function replicarObrigatoriedadeItensFilhos($params) {
        $sql = "UPDATE FCT_ITCOMPSERVICO
                   SET IDSERVOBRIGCOMP = :IDSERVOBRIGCOMP,
                       DTULTATU        = SYSDATE,
                       CDOPERULTATU    = :CDOPERULTATU,
                       NRORGULTATU     = :NRORGULTATU
                 WHERE NRCOMPSERVICO   = :NRCOMPSERVICO
                   AND IDELEMENTO     IN (SELECT IDELEMENTO
                                            FROM FCT_ITCOMPSERVICO
                                           WHERE IDELEMENTO         <> :IDELEMENTO
                                             AND NRCOMPSERVICO       = :NRCOMPSERVICO
                                           START WITH IDELEMENTO     = :IDELEMENTO
                                         CONNECT BY PRIOR IDELEMENTO = IDELEMENTOPAI)";
        
        $this->connection->executeStatement($sql, $params);
    }

    public function updateElementoServico($params)
    {
        $sql = "UPDATE FCT_ITCOMPSERVICO
                   SET
                        NRCOMPSERVICO       = :NRCOMPSERVICO,
                        IDSOMAVALORSUP      = :IDSOMAVALORSUP,
                        IDSERVOBRIGCOMP     = :IDSERVOBRIGCOMP,
                        IDELEMENTO          = :IDELEMENTO,
                        IDELEMENTOPAI       = :IDELEMENTOPAI,
                        IDTIPOELEMENTO      = :IDTIPOELEMENTO,
                        IDAPARECEPROPOSTA   = :IDAPARECEPROPOSTA,
                        IDATIVO             = :IDATIVO,
                        NRORG               = :NRORG,
                        NRORGULTATU         = :NRORG,
                        CDOPERULTATU        = :CDOPERULTATU,
                        DTULTATU            = SYSDATE
                WHERE NRITCOMPSERVICO   = :NRITCOMPSERVICO";

        $this->connection->executeStatement($sql, $params);
    }

    /**
     * Exclui o serviço da composição junto com seus filhos.
     */
    public function deleteItemComposicaoEmCascata(array $params) {
        $sql = "DELETE FROM FCT_ITCOMPSERVICO
                 WHERE NRITCOMPSERVICO IN (
                      SELECT NRITCOMPSERVICO FROM (
                             SELECT *
                               FROM FCT_ITCOMPSERVICO
                              WHERE NRCOMPSERVICO = :NRCOMPSERVICO
                          )
                        CONNECT BY PRIOR IDELEMENTO = IDELEMENTOPAI
                          START WITH IDELEMENTO     = :IDELEMENTO
                   )";
        
        $this->connection->executeStatement($sql, $params);
    }
}