<?php

namespace SM\Factory;

use Doctrine\ORM\EntityManager;

class SmNextDocumentoCdaFactory
{
    protected $connection;
    
    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
    }
    
    /**
     * Exclui os documentos temporários a ser utilizado para gerar o relatório.
     */
    public function excluirDocumentosTemporarios(array $params)
    {
        $sql = "DELETE FROM FCT_TMP_DOCUMENTOORCA
                 WHERE NRSEQORCAMENTO = :NRSEQORCAMENTO";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Retorna o documento da proposta para o escopo informado.
     */
    public function getDocumentoCDA($parametros)
    {
        $sql = "SELECT * FROM MODELODOCCDAIDI
                    JOIN SERVPRESTDOC
                    ON MODELODOCCDAIDI.NRMODELODOCCDA = SERVPRESTDOC.NRMODELODOCCDA
                    WHERE SERVPRESTDOC.CDEMPRESA = :CDEMPRESA
                                    AND SERVPRESTDOC.CDFUNC = :CDFUNC
                                    AND SERVPRESTDOC.CDCLIENTE = :CDCLIENTE
                                    AND SERVPRESTDOC.DTSERVPREST = :DTSERVPREST
                                    AND SERVPRESTDOC.NRSERVPREST = :NRSERVPREST";
        
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }
    
     /**
     * Retorna o representante do cliente da CDA.
     */
    public function getDadosRepresentanteCliCDA($parametros)
    {
        $sql = "SELECT
                    INTECLIE.CDINTECLIE,
                    INTECLIE.NMINTECLIE,
                    INTECLIE.NRORG,
                    INTECLIE.CDCLIENTE,
                    INTECLIE.DSEMAILICLIE,
                    INTECLIE.NRCPFICLIE,
                    INTECLIE.NRFAXINCLIE,
                    INTECLIE.NRTEL1INCLIE
                FROM
                    INTECLIE
                WHERE
                    CDCLIENTE = :CDCLIENTE
                    AND CDINTECLIE =:CDINTECLIE";
        
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    /**
     * Retorna os dados principais da proposta.
     */
    public function getDadosCDA($parametros)
    {
        $sql = "SELECT
                        SERVPREST.CDEMPRESA,
                        SERVPREST.CDFUNC,
                        SERVPREST.CDCLIENTE,
                        SERVPREST.CDINTECLIE,
                        CLIENTE.NMRAZSOCCLIE,
                        CLIENTE.NMFANTCLIE,
                        SERVPREST.NRLICENCAUSO,
                        LICENCAUSO.NMFANTLUSO,
                        TO_CHAR(SERVPREST.DTSERVPREST, 'DD/MM/YYYY') DTSERVPREST,
                        FCT_ORCAMENTO.NRORCAMENTO AS PROPOSTA,
                        SERVPREST.NRPROPOSFAS || '-' || SERVPREST.NRVERPROPFAS AS PROPOSTACOT,
                        SERVPREST.CDTIPOSERV,
                        TIPOSERV.NMTIPOSERV,
                        SERVPREST.NMINTCLIENTE,
                        SERVPREST.QTHRTOTAL,
                        SERVPREST.QTHRATIVIDADEFAT,
                        SERVPREST.QTHRDESLOCAMENTOFAT,
                        TO_CHAR(SERVPREST.DTENTRADA, 'HH24:MI') DTENTRADA,
                        TO_CHAR(SERVPREST.DTSAIDA, 'HH24:MI') DTSAIDA,
                        SERVPREST.HRAUSENCIAALMOCO,
                        SERVPREST.CDPROJETO,
                        SERVPREST.NRSERVPREST,
                        PROJETO.NMPROJETO,
                        DESCSERVPRES.DSSERVPREST,
                        FUNCIONARIO.NMFUNC
                    FROM SERVPREST
                    JOIN CLIENTE 
                    ON CLIENTE.CDCLIENTE = SERVPREST.CDCLIENTE
                    LEFT JOIN LICENCAUSO 
                    ON LICENCAUSO.NRLICENCAUSO = SERVPREST.NRLICENCAUSO
                    AND LICENCAUSO.CDCLIENTE = SERVPREST.CDCLIENTE
                    LEFT JOIN PROJETO 
                    ON PROJETO.CDPROJETO = SERVPREST.CDPROJETO
                    JOIN TIPOSERV 
                    ON TIPOSERV.CDTIPOSERV = SERVPREST.CDTIPOSERV
                    JOIN DESCSERVPRES 
                    ON DESCSERVPRES.CDEMPRESA = SERVPREST.CDEMPRESA
                    AND DESCSERVPRES.CDFUNC = SERVPREST.CDFUNC
                    AND DESCSERVPRES.CDCLIENTE = SERVPREST.CDCLIENTE
                    AND DESCSERVPRES.DTSERVPREST = SERVPREST.DTSERVPREST
                    AND DESCSERVPRES.NRSERVPREST = SERVPREST.NRSERVPREST
                    JOIN FUNCIONARIO
                    ON FUNCIONARIO.CDFUNC = SERVPREST.CDFUNC
                    LEFT JOIN PROPOSTAFAS
                    ON PROPOSTAFAS.NRPROPOSFAS = SERVPREST.NRPROPOSFAS
                    AND PROPOSTAFAS.NRVERPROPFAS = SERVPREST.NRVERPROPFAS
                    LEFT JOIN FCT_REFEORCACONT
                    ON FCT_REFEORCACONT.NRSEQCONTRATO = PROPOSTAFAS.NRSEQCONTRATO
                    LEFT JOIN FCT_ORCAMENTO 
                    ON FCT_ORCAMENTO.NRSEQORCAMENTO = FCT_REFEORCACONT.NRSEQORCAMENTO
                    WHERE SERVPREST.CDEMPRESA = :CDEMPRESA
                    AND SERVPREST.CDFUNC = :CDFUNC
                    AND SERVPREST.CDCLIENTE = :CDCLIENTE
                    AND SERVPREST.DTSERVPREST = :DTSERVPREST
                    AND SERVPREST.NRSERVPREST = :NRSERVPREST";
        
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function getParticipantesCda($parametros)
    {
        $sql = "SELECT DISTINCT ROWNUM AS IND, DSNOMEPARTICIP, DSSETORPARTICIP
                    FROM SERVPRESTPARTICIP
                    WHERE CDEMPRESA = :CDEMPRESA
                    AND CDFUNC = :CDFUNC
                    AND CDCLIENTE = :CDCLIENTE
                    AND DTSERVPREST = TO_DATE(:DTSERVPREST, 'DD/MM/YYYY')
                    AND NRSERVPREST = :NRSERVPREST";
                
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function getHorasNaoFaturadas($parametros)
    {
        $sql = "SELECT JUSTHORASNAOFATUCDA.NRJUSTHORASNAOFATUCDA, JUSTHORASNAOFATUCDA.DSJUSTHORASNAOFATUCDA, 
                    JUSTHORASNAOFATUCDA.TAGJUSTHORASNAOFATUCDA, SERVPRESTJUSTHRNFAT.QTHORASNAOFATU, SERVPRESTJUSTHRNFAT.NRSERVPRESTJUSTHRNFAT
                    FROM JUSTHORASNAOFATUCDA
                    LEFT JOIN SERVPRESTJUSTHRNFAT
                    ON SERVPRESTJUSTHRNFAT.NRJUSTHORASNAOFATUCDA = JUSTHORASNAOFATUCDA.NRJUSTHORASNAOFATUCDA
                    WHERE SERVPRESTJUSTHRNFAT.CDEMPRESA = :CDEMPRESA
                    AND SERVPRESTJUSTHRNFAT.CDFUNC = :CDFUNC
                    AND SERVPRESTJUSTHRNFAT.CDCLIENTE = :CDCLIENTE
                    AND SERVPRESTJUSTHRNFAT.DTSERVPREST = TO_DATE(:DTSERVPREST)
                    AND SERVPRESTJUSTHRNFAT.NRSERVPREST = :NRSERVPREST";
        
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function getCodigoItensParticipantes($parametros)
    {
        $sql = "SELECT DISTINCT ROWNUM AS IND,
                       PARTICIP.*
                  FROM (SELECT DSNOMEPARTICIP, DSSETORPARTICIP
                    FROM SERVPRESTPARTICIP
                    WHERE CDEMPRESA = :CDEMPRESA
                    AND CDFUNC = :CDFUNC
                    AND CDCLIENTE = :CDCLIENTE
                    AND DTSERVPREST = TO_DATE(:DTSERVPREST, 'DD/MM/YYYY')
                    AND NRSERVPREST = :NRSERVPREST) PARTICIP";
                   
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    // Dados para o documento da CDA utilizando a nova estrutura
    
    /**
     * Retorna os dados principais da CDA.
     */
    public function getDadosApropCDA($parametros)
    {
        $sql = "SELECT
                    APRO.NRAPROPAGENDAMENTO,
                    APRO.NRTIPOLANCAMENTOCDA,
                    TPLANCAMENTO.NMTIPOLANCAMENTOCDA,
                    PARCCONTRATO.NMPRINCIPALPARC,
                    PARCCONTRATO.NRPARCNEGOCIO AS PARCNEGOCIOCLIENTE,
                    APRO.IDSITUACAOCDA,
                    APRO.NRRECURSOSAGENDCONT,
                    APRO.NRPARCNEGRECURSO,
                    APRO.NRSEQINTERLOCUTOR,
                    INTERLOCUTOR.NMINTERLOCUTOR,
                    APRO.DSCDA,
                    ORCA.NRSEQORCAMENTO,
                    ORCA.NRORCAMENTO,
                    APRO.DTAPROPRIACAO,
                    TO_CHAR(APRO.DTAPROPRIACAO, 'DD/MM/YYYY') AS DTAPROPRIACAO,
                    HORADEC_HORACHAR(APRO.QTHORASAPROPRIADAS) AS QTHORAS,
                    APRO.IDSTATUSAPROPRIACAO,
                    APRO.QTHORASAPROPRIADAS,
                    TO_CHAR(APRO.DTENTRADA, 'HH24:MI') AS DTENTRADA,
                    TO_CHAR(APRO.DTSAIDA, 'HH24:MI') AS DTSAIDA,
                    APRO.QTHRDESLOCAMENTOFAT,
                    APRO.QTHRTOTAL,
                    APRO.HRAUSENCIAALMOCO,
                    APRO.DSJUSTHRNAOFATURADA,
                    APRO.NRPARCINTERLOCUTOR,
                    APRO.IDORIGEMAGEND,
                    APRO.NRORG,
                    APRO.IDATIVO,
                    APRO.DTINCLUSAO,
                    APRO.NRORGINCLUSAO,
                    APRO.CDOPERINCLUSAO,
                    APRO.DTULTATU,
                    APRO.NRORGULTATU,
                    APRO.CDOPERULTATU,
                    AGENDA.NRSEQCONTRATO,
                    AGENDA.NRAGENDRECURSOCONT,
                    CONT.NRCONTRATOFCT,
                    CONT.NRCONTRATOFCT || ' - ' || CONT.DSREFERENCIACONT AS AGENDAMENTO,
                    SERVICO.NMSERVORCA,
                    APROPDOC.NRAPROPAGENDAMENTODOC,
                    APROPDOC.IDDOCUMCERTSIGN,
                    APROPDOC.IDSITASSINATURA,
                    APROPDOC.DTASSINATURA,
                    APROPDOC.DTLEMBRETEASSINATURA,
                    APRO.NRSEQEVOLCONT,
                    APRO.NRMOTIVOEXCEFREQ,
                    JUSTIFICATIVA.NMMOTIVOEXCEFREQ,
                    CLI.CDCLIENTE,
                    CLI.NMRAZSOCCLIE,
                    CLI.NMFANTCLIE,
                    OPER.CDOPERADOR,
                    OPER.NMOPERADOR
                FROM FCT_APROPAGENDAMENTO APRO
                LEFT JOIN FCT_RECURSOSAGENDCONT RECURSO ON APRO.NRRECURSOSAGENDCONT = RECURSO.NRRECURSOSAGENDCONT
                LEFT JOIN FCT_AGENDRECURSOCONT AGENDA ON RECURSO.NRAGENDRECURSOCONT = AGENDA.NRAGENDRECURSOCONT
                LEFT JOIN FCT_TIPOLANCAMENTOCDA TPLANCAMENTO ON APRO.NRTIPOLANCAMENTOCDA = TPLANCAMENTO.NRTIPOLANCAMENTOCDA
                LEFT JOIN GPE_MOTIVOEXCEFREQ JUSTIFICATIVA ON APRO.NRMOTIVOEXCEFREQ = JUSTIFICATIVA.NRMOTIVOEXCEFREQ
                LEFT JOIN OPERADOR OPER ON RECURSO.NRPARCNEPESSOA = OPER.NRPARCNEGOCIO
                LEFT JOIN PARCNEGOCIO PARCRECURSO ON RECURSO.NRPARCNEPESSOA = PARCRECURSO.NRPARCNEGOCIO
                LEFT JOIN FCT_CONTRATO CONT ON AGENDA.NRSEQCONTRATO = CONT.NRSEQCONTRATO
                LEFT JOIN FCT_REFEORCACONT REFCONT ON CONT.NRSEQCONTRATO = REFCONT.NRSEQCONTRATO
                LEFT JOIN FCT_ORCAMENTO    ORCA ON REFCONT.NRSEQORCAMENTO = ORCA.NRSEQORCAMENTO
                LEFT JOIN PARCNEGOCIO PARCCONTRATO ON CONT.NRPARCNEGOCIO = PARCCONTRATO.NRPARCNEGOCIO
                LEFT JOIN CRM_INTERLOCUTOR INTERLOCUTOR ON APRO.NRSEQINTERLOCUTOR = INTERLOCUTOR.NRSEQINTERLOCUTOR
                LEFT JOIN CLIENTE CLI ON PARCCONTRATO.NRPARCNEGOCIO = CLI.NRPARCNEGOCIO
                LEFT JOIN FCT_ITEMCONTRATO ITEM ON CONT.NRSEQCONTRATO = ITEM.NRSEQCONTRATO 
                        AND RECURSO.NRITEMOCONTRATO = ITEM.NRITEMOCONTRATO
                LEFT JOIN FCT_SERVICOORCA SERVICO ON ITEM.CDSERVORCA = SERVICO.CDSERVORCA
                LEFT JOIN FCT_APROPAGENDAMENTODOC APROPDOC ON APRO.NRAPROPAGENDAMENTO = APROPDOC.NRAPROPAGENDAMENTO
                WHERE APRO.NRAPROPAGENDAMENTO = :NRAPROPAGENDAMENTO";
        
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function getParticipantesApropCda($parametros)
    {
        $sql = "SELECT DISTINCT ROWNUM AS IND, DSNOMEPARTICIP, DSSETORPARTICIP
                    FROM FCT_PARTICIPANTEAPROP
                    WHERE NRAPROPAGENDAMENTO = :NRAPROPAGENDAMENTO";
                
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }
    
    public function getCodigoItensParticipantesAprop($parametros)
    {
        $sql = "SELECT DISTINCT ROWNUM AS IND,
                       PARTICIP.*
                  FROM (SELECT DSNOMEPARTICIP, DSSETORPARTICIP
                    FROM FCT_PARTICIPANTEAPROP
                    WHERE NRAPROPAGENDAMENTO = :NRAPROPAGENDAMENTO) PARTICIP";
                   
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function getHorasNaoFaturadasApropCda($parametros)
    {
        $sql = "SELECT JUSTHORASNAOFATUCDA.NRJUSTHORASNAOFATUCDA, JUSTHORASNAOFATUCDA.DSJUSTHORASNAOFATUCDA, 
                    JUSTHORASNAOFATUCDA.TAGJUSTHORASNAOFATUCDA, FCT_APROPJUSTHRNFAT.QTHORASNAOFATU, FCT_APROPJUSTHRNFAT.NRAPROPJUSTHRNFAT
                    FROM JUSTHORASNAOFATUCDA
                    LEFT JOIN FCT_APROPJUSTHRNFAT
                    ON FCT_APROPJUSTHRNFAT.NRJUSTHORASNAOFATUCDA = JUSTHORASNAOFATUCDA.NRJUSTHORASNAOFATUCDA
                    WHERE FCT_APROPJUSTHRNFAT.NRAPROPAGENDAMENTO = :NRAPROPAGENDAMENTO";
        
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    /**
     * Retorna o documento da CDA para o Idioma informado.
     */
    public function getDocumentoApropCDA($parametros)
    {
        $sql = "SELECT * FROM MODELODOCCDAIDI
                    JOIN FCT_APROPAGENDAMENTODOC
                    ON MODELODOCCDAIDI.NRMODELODOCCDA = FCT_APROPAGENDAMENTODOC.NRMODELODOCCDA
                    WHERE FCT_APROPAGENDAMENTODOC.NRAPROPAGENDAMENTO = :NRAPROPAGENDAMENTO";
        
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    /**
     * Retorna o representante do cliente da CDA.
     */
    public function getDadosInterlocutorCDA($parametros)
    {
        $sql = "SELECT 
                    NRSEQINTERLOCUTOR, 
                    NMINTERLOCUTOR, 
                    EMAILINTERLOC, 
                    NRCPFINTERLOC
                FROM CRM_INTERLOCUTOR 
                    WHERE NRSEQINTERLOCUTOR = :NRSEQINTERLOCUTOR";
        
        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }
}
