import { Utilities } from '@/util/Utilities';
import {
    Button, DateRange, Frame, Modal, ModalService, Tabs,
} from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { DashboardGrid, IDashboardGridItem } from '@zeedhi/zd-dashboard-grid-common';
import { ECharts } from '@zeedhi/zd-echarts';

export class NewDashboardController {
    public modalFiltro!: Modal;

    public modalVisibilidade!: Modal;

    public textFooterRange: string = 'Filtro de Tempo: Últimos 30 dias';
    
    public filtroRangeInicio: string = '';

    public filtroRangeFinal: string = '';
    
    public queryStringFiltro: string = `?DTRANGEINICIO=${this.filtroRangeInicio}&DTRANGEFINAL=${this.filtroRangeFinal}`;
    
    public teste() {
        console.warn('Dashboard Controller');
    }

    public async criarModais() {
        this.modalFiltro = await ModalService.createFromJson('newDashboard/modal/sm_filtro', true);
        this.modalVisibilidade = await ModalService.createFromJson('newDashboard/modal/sm_visibilidade', true);
    }

    public async abrirModalVisibilidade() {
        await this.modalVisibilidade.show();
        await this.loadHiddenCards();
    }

    public fecharModalVisibilidade() {
        this.modalVisibilidade.hide();
    }

    public async abrirModalFiltro() {
        this.modalFiltro.show();
    }

    public fecharModalFiltro() {
        this.modalFiltro.hide();
    }

    public aplicarFiltro() {
        const dateRange = Metadata.getInstance<DateRange>('periodo-busca-dashboard');
        const [filtroRangeInicio, filtroRangeFinal] = dateRange.value;

        if (this.filtroRangeFinal === filtroRangeFinal && this.filtroRangeInicio === filtroRangeInicio) {
            this.fecharModalFiltro();
            return;
        }

        this.filtroRangeInicio = filtroRangeInicio || '';
        this.filtroRangeFinal = filtroRangeFinal || '';

        this.queryStringFiltro = `?DTRANGEINICIO=${this.filtroRangeInicio}&DTRANGEFINAL=${this.filtroRangeFinal}`;

        this.montarFooterRange();
        this.fecharModalFiltro();
        this.reloadAllDashs();
    }

    public montarFooterRange() {
        if (this.filtroRangeFinal === '' || this.filtroRangeInicio === '') {
            this.textFooterRange = 'Filtro de Tempo: Últimos 30 dias';
            // this.titleGrid = 'Facilities | Tempo Todo';
            return;
        }
        this.textFooterRange = `Filtro de Tempo: Mostrando de ${this.filtroRangeInicio} a ${this.filtroRangeFinal}`;
        // this.titleGrid = `Facilities | ${this.filtroRangeInicio}     -     ${this.filtroRangeFinal}`;
    }

    public restaurarCards() {
        const tabs = Metadata.getInstance<Tabs>('tabs-dash');

        if (tabs.activeTab === 0) {
            localStorage.removeItem('@ZdDashboardGrid/dashboard-propostas');
            localStorage.removeItem('@ZdDashboardGrid/hideCards/dashboard-propostas');
            this.reloadDash();
        }
        
        if (tabs.activeTab === 1) {
            localStorage.removeItem('@ZdDashboardGrid/dashboard-contratos');
            localStorage.removeItem('@ZdDashboardGrid/hideCards/dashboard-contratos');
            this.reloadDash();
        }

        if (tabs.activeTab === 2) {
            localStorage.removeItem('@ZdDashboardGrid/dashboard-faturamentos');
            localStorage.removeItem('@ZdDashboardGrid/hideCards/dashboard-faturamentos');
            this.reloadDash();
        }
    }

    public reloadDash() {
        const tabs = Metadata.getInstance<Tabs>('tabs-dash');

        if (tabs.activeTab === 0) {
            const frame = Metadata.getInstance<Frame>('frame-dash-propostas');
            frame.reload();
        }
        
        if (tabs.activeTab === 1) {
            const frame = Metadata.getInstance<Frame>('frame-dash-contratos');
            frame.reload();
        }

        if (tabs.activeTab === 2) {
            const frame = Metadata.getInstance<Frame>('frame-dash-faturamentos');
            frame.reload();
        }
    }

    public reloadAllDashs() {
        const dashPropostas = Metadata.getInstance<Frame>('frame-dash-propostas');
        dashPropostas.reload();

        const dashVistorias = Metadata.getInstance<Frame>('frame-dash-contratos');
        dashVistorias.reload();

        const dashFaturamentos = Metadata.getInstance<Frame>('frame-dash-faturamentos');
        dashFaturamentos.reload();
    }

    public loadHiddenCards() {
        const hiddenCards = Metadata.getInstance<TekGrid>('grid-cards-escondidos');
        const cardsOcultosPropostas = localStorage.getItem('@ZdDashboardGrid/hideCards/dashboard-propostas');
        const cardsOcultosContratos = localStorage.getItem('@ZdDashboardGrid/hideCards/frame-dash-contratos');
        const cardsOcultosFaturamentos = localStorage.getItem('@ZdDashboardGrid/hideCards/dashboard-faturamentos');

        const tarefasData = cardsOcultosPropostas ? JSON.parse(cardsOcultosPropostas) : [];
        const contratosData = cardsOcultosContratos ? JSON.parse(cardsOcultosContratos) : [];
        const faturamentosData = cardsOcultosFaturamentos ? JSON.parse(cardsOcultosFaturamentos) : [];

        const cardsOcultos = [...tarefasData, ...contratosData, ...faturamentosData];

        hiddenCards.datasource.updateData(cardsOcultos);
    }

    public visibilidadeCard({ row }: any) {
        const nome: string = `${row.name.split('-').pop()?.trim().toLowerCase() ?? ''}s`;

        const dashboard = this.getDashboard(nome);
        if (!dashboard) {
            Utilities.showErrorMessage('Erro ao esconder Card');
            return;
        }

        dashboard.toggleVisibility(row);
        this.loadHiddenCards();
    }
    
    public getDashboard(nome: string) {
        switch (nome) {
            case 'propostas':
                return Metadata.getInstance<DashboardGrid>('dashboard-propostas');
            case 'contratos':
                return Metadata.getInstance<DashboardGrid>('dashboard-contratos');
            case 'faturamentos':
                return Metadata.getInstance<DashboardGrid>('dashboard-faturamentos');
            default:
                return null;
        }
    }

    public async printCard({ component }:any): Promise<void> {
        const nome = component.name.split('-');
        const nomeFormatado = nome.length > 1 ? nome[nome.length - 2].trim() : '';
        const dashboardInstance = this.getDashboard(nomeFormatado);

        if (!dashboardInstance) {
            Utilities.showErrorMessage('Erro ao recuperar dashboardInstance');
            return;
        }

        const cardId = `DashboardGridFrame-${dashboardInstance.currentCardId}-dashboard-${nomeFormatado}`;
        const cardElement = document.getElementById(cardId);

        if (!cardElement) {
            Utilities.showErrorMessage('Erro ao recuperar cardElement');
            return;
        }

        const elementCanva = cardElement.querySelector('canvas');

        if (elementCanva) {
            const originalWidth = elementCanva.width;
            const originalHeight = elementCanva.height;

            const tempCanvas = document.createElement('canvas');
            tempCanvas.width = originalWidth;
            tempCanvas.height = originalHeight;
            const tempCtx = tempCanvas.getContext('2d');

            if (tempCtx) {
                tempCtx.fillStyle = 'white';
                tempCtx.fillRect(0, 0, originalWidth, originalHeight);

                tempCtx.drawImage(elementCanva, 0, 0);

                const canvas2 = tempCanvas.toDataURL();
                this.downloadImage(canvas2, 'chartPrint.png');
            }
        }
    }

    private downloadImage(imageData: string, filename: string): void {
        const xhr = new XMLHttpRequest();
        xhr.responseType = 'blob';

        xhr.onload = () => {
            const blob = xhr.response;
            const url = window.URL.createObjectURL(blob);
            const a = document.createElement('a');

            a.href = url;
            a.download = filename;
            a.style.display = 'none';
            document.body.appendChild(a);
            a.click();
            a.remove();

            window.URL.revokeObjectURL(url);
        };

        xhr.open('GET', imageData);
        xhr.send();
    }

    public posicaoLegenda({ component }: any) {
        const nome = component.name.split('-');
        const dashboardInstance = this.getDashboard(nome.length > 1 ? nome[nome.length - 2].trim() : '');

        if (dashboardInstance) {
            const pathComponent = dashboardInstance.getFrameProps(dashboardInstance.currentCardId!);
            const nameComponent = pathComponent.path?.substring(pathComponent.path.lastIndexOf('/') + 1);
            const chart = Metadata.getInstance<ECharts>(`${nameComponent}`);
            const NewOptions = { ...chart.options };

            if (component.name.split('-').slice(0, 2).join('-') === 'superior-esquerdo') {
                NewOptions.legend.right = '80%';
                NewOptions.legend.top = '5%';
            }
            if (component.name.split('-').slice(0, 2).join('-') === 'superior-direito') {
                NewOptions.legend.right = '10%';
                NewOptions.legend.top = '5%';
            }
            if (component.name.split('-').slice(0, 2).join('-') === 'inferior-esquerdo') {
                NewOptions.legend.right = '10%';
                NewOptions.legend.top = '80%';
            }
            if (component.name.split('-').slice(0, 2).join('-') === 'inferior-direito') {
                NewOptions.legend.right = '10%';
                NewOptions.legend.top = '80%';
            }
            if (component.name.split('-').slice(0, 2).join('-') === 'esconder-legendas') {
                NewOptions.legend.show = !(NewOptions.legend.show);
            }

            chart.updateChart(NewOptions);
        }
    }

    public async esconderCard({ component }: any): Promise<void> {
        const nome = component.name.split('-');
        const dashboardInstance = this.getDashboard(nome.length > 1 ? nome[nome.length - 2].trim() : '');

        if (!dashboardInstance) {
            Utilities.showErrorMessage('Erro ao Recuperar a instancia do dashboard');
            return;
        }

        const { cards } = dashboardInstance;
        cards.forEach((card: IDashboardGridItem): void => {
            if (card.i === dashboardInstance.currentCardId) {
                dashboardInstance.showToggleVisibilityCardDialog(card);
            }
        });
    }

    public fullscreen({ component }: any) {
        const nome = component.name.split('-');
        const nomeFormatado = nome.length > 1 ? nome[nome.length - 2].trim() : '';
        const dashboardInstance = this.getDashboard(nomeFormatado);

        if (dashboardInstance) {
            const botaoFullScreen = Metadata.getInstance<Button>(`fullScreen-card-${nomeFormatado}-${dashboardInstance.currentCardId}`);
            const botaoEsconder = Metadata.getInstance<Button>(`esconder-card-${nomeFormatado}-${dashboardInstance.currentCardId}`);
            const card: string = dashboardInstance.currentCardId!;

            dashboardInstance.toggleFullScreen(card);
            if (botaoFullScreen.iconName === 'mdi-fullscreen') {
                botaoFullScreen.iconName = 'mdi-fullscreen-exit';
                botaoEsconder.disabled = true;
            } else {
                botaoFullScreen.iconName = 'mdi-fullscreen';
                botaoEsconder.disabled = false;
            }
        }
    }

    public getQtdPropostasAceitasPorEscopo() {
        const grafico = Metadata.getInstance<ECharts>('sm_card_prop_qtd_pro_ace_x_esc_fat');
        const NewOptions = { ...grafico.options };

        Http.get(`/getQtdPropostasAceitasPorEscopo${this.queryStringFiltro}`).then((response: any) => {
            const propostas = response.data.data;

            propostas.forEach((element: any) => {
                NewOptions.series[0].data.push({
                    name: element.NMTIPOORCAMENTO,
                    value: element.QUANTIDADE,
                });
            });
            grafico.updateChart(NewOptions);
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public getValorPropostasAceitasPorEscopo() {
        const grafico = Metadata.getInstance<ECharts>('sm_card_prop_valor_pro_ace_x_esc_fat');
        const NewOptions = { ...grafico.options };

        Http.get(`/getValorPropostasAceitasPorEscopo${this.queryStringFiltro}`).then((response: any) => {
            const propostas = response.data.data;

            propostas.forEach((element: any) => {
                NewOptions.series[0].data.push({
                    name: element.NMTIPOORCAMENTO,
                    value: element.VALOR,
                });
            });
            grafico.updateChart(NewOptions);
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public getValorPropostasAceitasPorParceiro() {
        const grafico = Metadata.getInstance<ECharts>('sm_card_prop_pro_ace_x_parceiro');
        const NewOptions = { ...grafico.options };

        Http.get(`/getValorPropostasAceitasPorParceiro${this.queryStringFiltro}`).then((response: any) => {
            const propostas = response.data.data;

            propostas.forEach((element: any) => {
                NewOptions.xAxis.data.push(element.NMSECUNDARIPARC);
                NewOptions.series[0].data.push(element.VALOR);
            });
            grafico.updateChart(NewOptions);
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public getContratosAprovadosPorEscopo() {
        const grafico = Metadata.getInstance<ECharts>('sm_card_cont_aprovados_x_esc_fat');
        const NewOptions = { ...grafico.options };

        Http.get(`/getContratosAprovadosPorEscopo${this.queryStringFiltro}`).then((response: any) => {
            const propostas = response.data.data;

            propostas.forEach((element: any) => {
                NewOptions.series[0].data.push({
                    name: element.NMTIPOORCAMENTO,
                    value: element.QUANTIDADE,
                });
            });
            grafico.updateChart(NewOptions);
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public getContratosRescindidosPorEscopo() {
        const grafico = Metadata.getInstance<ECharts>('sm_card_cont_rescindidos_x_esc_fat');
        const NewOptions = { ...grafico.options };

        Http.get(`/getContratosRescindidosPorEscopo${this.queryStringFiltro}`).then((response: any) => {
            const propostas = response.data.data;

            propostas.forEach((element: any) => {
                NewOptions.series[0].data.push({
                    name: element.NMTIPOORCAMENTO,
                    value: element.QUANTIDADE,
                });
            });
            grafico.updateChart(NewOptions);
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public getContratosRenegociadosPorEscopo() {
        const grafico = Metadata.getInstance<ECharts>('sm_card_cont_renegociados_x_esc_fat');
        const NewOptions = { ...grafico.options };

        Http.get(`/getContratosRenegociadosPorEscopo${this.queryStringFiltro}`).then((response: any) => {
            const propostas = response.data.data;

            propostas.forEach((element: any) => {
                NewOptions.series[0].data.push({
                    name: element.NMTIPOORCAMENTO,
                    value: element.QUANTIDADE,
                });
            });
            grafico.updateChart(NewOptions);
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }
    
    public getFaturamentosValorFatPorParceiro() {
        const grafico = Metadata.getInstance<ECharts>('sm_card_fat_valor_fat_x_parceiro');
        const NewOptions = { ...grafico.options };

        Http.get(`/getFaturamentosValorFatPorParceiro${this.queryStringFiltro}`).then((response: any) => {
            const faturamentos = response.data.data;

            faturamentos.forEach((element: any) => {
                NewOptions.series[0].data.push({
                    name: element.NMSECUNDARIPARC,
                    value: element.VALOR,
                });
            });
            grafico.updateChart(NewOptions);
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public getFaturamentosValorFatPorEscopo() {
        const grafico = Metadata.getInstance<ECharts>('sm_card_fat_valor_fat_x_esp_fat');
        const NewOptions = { ...grafico.options };

        Http.get(`/getFaturamentosValorFatPorEscopo${this.queryStringFiltro}`).then((response: any) => {
            const faturamentos = response.data.data;

            faturamentos.forEach((element: any) => {
                NewOptions.series[0].data.push({
                    name: element.NMTIPOORCAMENTO,
                    value: element.VALOR,
                });
            });
            grafico.updateChart(NewOptions);
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public getFaturamentosValorFatPorItemContrato() {
        const grafico = Metadata.getInstance<ECharts>('sm_card_fat_valor_fat_x_item_cont');
        const NewOptions = { ...grafico.options };

        Http.get(`/getFaturamentosValorFatPorItemContrato${this.queryStringFiltro}`).then((response: any) => {
            const faturamentos = response.data.data;

            faturamentos.forEach((element: any) => {
                NewOptions.series[0].data.push({
                    name: element.NMITEMCONTRATO,
                    value: element.VALOR,
                });
            });
            grafico.updateChart(NewOptions);
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public getFaturamentosValorFaturadoPorMes() {
        const grafico = Metadata.getInstance<ECharts>('sm_card_fat_valor_faturamento_mes');
        const NewOptions = { ...grafico.options };

        Http.get('/getFaturamentosValorFaturadoPorMes').then((response: any) => {
            const faturamentos = response.data.data;

            faturamentos.forEach((element: any) => {
                NewOptions.xAxis.data.push(element.MES);
                NewOptions.series[0].data.push(element.VALOR);
            });
            grafico.updateChart(NewOptions);
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public setFilterGridContratoFaturamentoPendente() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-contratos-faturamento-pendente');
        grid.datasource.setFilter({
            DATA_RANGE: this.queryStringFiltro,
        });
    }
}
