import {
    Http, IEventParam, Metadata,
} from '@zeedhi/core';
import {
    Button,
    DialogService, GridEditable, IGridEvent, LoadingService,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class Sm00024000TabDespesaDeMaoDeObraController {
    public naoEditar() {
        const grid = Metadata.getInstance<TekGrid>('grid-desp-mao-obra');
        grid.editing = false;
    }

    public addNewRow() {
        const grid = Metadata.getInstance<TekGrid>('grid-desp-mao-obra');
        const newId = 0;
        grid.addNewRow({ NRMAOOBRACONTCTBL: newId }, 'start');
    }

    public adicionar_desp_mao_obra({ event }: IGridEvent) {
        event?.preventDefault();
        const grid = Metadata.getInstance<TekGrid>('grid-desp-mao-obra');
        const prc = grid.getEditedRows();

        if (Utilities.validateGrid(grid)) {
            LoadingService.show();
            Http.post('/adicionar_desp_mao_obra', prc).then(() => {
                grid.cancelEditedRows();
                grid.reload();
                Utilities.showSuccessMessage();
                grid.datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public afterModify() {
        const grid = Metadata.getInstance<TekGrid>('grid-desp-mao-obra');
        grid.datasource.get();
        grid.columns.forEach((column) => {
            column.editable = false;
        });
    }

    public excluir_desp_mao_obra({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'Sm00024000TabDespesaDeMaoDeObraController.aposConfirmarDelete');
    }

    public aposConfirmarDelete() {
        const grid = Metadata.getInstance<TekGrid>('grid-desp-mao-obra');
        LoadingService.show();
        Http.post('/excluir_desp_mao_obra', {
            DMO: grid.selectedRows.map((element: any) => element.NRMAOOBRACONTCTBL),
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            grid.selectedRows = [];
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public canEditRowIsNotInlineEditing = true;

    public canEditRowOnInlineEdit({ component }: IEventParam<GridEditable>) {
        this.canEditRowIsNotInlineEditing = !component.editing;
    }
    
    public async canEditRowCancelEditing({ component }: IEventParam<Button>) {
        const grid = component.parent as GridEditable;
        await grid.cancelEditedRows();
        
        this.canEditRowOnInlineEdit({ component: grid });
        grid.editingNewRows = false;
    }
    
    public async canEditRowSaveEditing({ component }: IEventParam<Button>) {
        const grid = component.parent as GridEditable;
        await grid.saveEditedRows();
        
        this.canEditRowOnInlineEdit({ component: grid });
        grid.editingNewRows = false;
    }
    
    public canEditRow({ row }: IEventParam<GridEditable>) {
        return row.id !== 1;
    }
}
